Package: BgeeDB
Type: Package
Title: Annotation and gene expression data retrieval from Bgee
        database. TopAnat, an anatomical entities Enrichment Analysis
        tool for UBERON ontology
Version: 2.38.1
Date: 2024-08-20
Author: Andrea Komljenovic [aut, cre], Julien Roux [aut, cre]
Maintainer: Julien Wollbrett <julien.wollbrett@unil.ch>, Julien Roux <julien.roux@unibas.ch>, Andrea Komljenovic  <andreakomljenovic@gmail.com>, Frederic Bastian <bgee@sib.swiss>
Description: A package for the annotation and gene expression
    data download from Bgee database, and TopAnat analysis: GO-like enrichment of anatomical terms,
    mapped to genes by expression patterns.
Encoding: UTF-8
Depends: R (>= 3.6.0), topGO, tidyr
Imports: R.utils, data.table, curl, RCurl, digest, methods, stats,
        utils, dplyr, RSQLite, graph, Biobase, zellkonverter, anndata,
        HDF5Array, bread
License: GPL-3 + file LICENSE
Roxygen: list(markdown = TRUE)
URL: https://github.com/BgeeDB/BgeeDB_R
BugReports: https://github.com/BgeeDB/BgeeDB_R/issues
VignetteBuilder: knitr
biocViews: Software, DataImport, Sequencing, GeneExpression,
        Microarray, GO, GeneSetEnrichment
Suggests: knitr, BiocStyle, testthat, rmarkdown, markdown
LazyLoad: yes
RoxygenNote: 7.3.2
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/BgeeDB
git_branch: RELEASE_3_23
git_last_commit: a460555
git_last_commit_date: 2026-04-29
Repository: Bioconductor 3.23
Date/Publication: 2026-04-29
Packaged: 2026-04-29 20:40:56 UTC; biocbuild
