# Generated by roxygen2: do not edit by hand

export(Bgee)
export(deleteLocalData)
export(deleteOldData)
export(formatData)
export(getAnnotation)
export(getCellProcessedData)
export(getData)
export(getIntegratedCalls)
export(getSampleProcessedData)
export(listBgeeRelease)
export(listBgeeSpecies)
export(loadTopAnatData)
export(makeTable)
export(topAnat)
exportClasses(Bgee)
import(Biobase)
import(HDF5Array)
import(RCurl)
import(RSQLite)
import(anndata)
import(bread)
import(curl)
import(data.table)
import(digest)
import(graph)
import(methods)
import(stats)
import(topGO)
import(utils)
import(zellkonverter)
importFrom(R.utils,gunzip)
importFrom(dplyr,"%>%")
importFrom(tidyr,spread_)
